#include <avr/io.h> 
#include <avr/interrupt.h>   
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "25664_screens.h"

// all on PORTC
#define	OLED_CD	  PC7   
#define	OLED_WR   PC6	
#define	OLED_RD   PC5	
#define	OLED_CS	  PC4
#define	OLED_RES	PC2

#define CLR_CD PORTC &= ~(1<<OLED_CD);
#define SET_CD PORTC |=  (1<<OLED_CD);

#define CLR_CS PORTC &= ~(1<<OLED_CS);
#define SET_CS PORTC |=  (1<<OLED_CS);

#define CLR_RESET PORTC &= ~(1<<OLED_RES);
#define SET_RESET PORTC |=  (1<<OLED_RES);

#define CLR_WR PORTC &= ~(1 << OLED_WR);   
#define SET_WR PORTC |=  (1 << OLED_WR);   
#define CLR_RD PORTC &= ~(1 << OLED_RD);   
#define SET_RD PORTC |=  (1 << OLED_RD);   

void delay(uint32_t twait)
{
	while (twait--)
		asm volatile ("nop");
}

void delayms(uint16_t us)
{
	while (us--)
		delay(1000);
}

void oled_cmd(uint8_t command)
{

	PORTA = command;	// set up data on bus
  CLR_CS;   // chip selected
  CLR_CD;   // command mode
	// clock WR
  CLR_WR;
  SET_WR;
  SET_CS;   // deselect chip
}

void oled_data(uint8_t data)
{
	PORTA = data;	// set up data on bus
  SET_CD;       // data mode
  CLR_CS;       // chip selected
	// clock WR
  CLR_WR;
  SET_WR;
  SET_CS;       // deselect chip
}

void reset_display(void)
{
	CLR_RESET;
  delay(10000);
  SET_RESET;
}

void oled_clr(uint8_t color)
{
	int i;
	oled_cmd (0x13); // Ram X start
	oled_cmd (0x00);  
	oled_cmd (0x14); // Ram Y start
	oled_cmd (0x00); 

	for (i=0;i<256*64;i++)
		oled_data(color);
}

//
// display a 16-level greyscale 256*64 pixel bitmap
//
void bmp16(uint8_t *p)
{
	int i,j;

	oled_cmd (0x15); // mode setup
	oled_cmd (0x03); // 16 level grey, inc x, inc y

	for (j=0;j<64;j++)
	{
		oled_cmd (0x13); // Ram X start
		oled_cmd (0x00); 
		oled_cmd (0x14); // Ram Y start
		oled_cmd (j); 

		for (i=0;i<128;i++)
		{
			uint8_t p1 = pgm_read_byte(p++);
			//oled_data (p1);	// if nibbles are located correctly
			oled_data( ( ( (p1 & 0x0f)) << 4) | ( (p1 >> 4)) ); // for nibbles that needs swapping
		}
	}
}

void disp_onoff(uint8_t mode)
{
	oled_cmd (0x00);
	oled_cmd (mode ? 1 : 0);
}

/*******************************************************************************************/
void init_OLED(void)
{
  reset_display();
  delayms(50);

  oled_cmd(0x01); // Initial Brightness Adjustment A
  oled_cmd(0xFF); // Initial adjustment A

  oled_cmd(0x02); // Initial Brightness Adjustment B
  oled_cmd(0xFF); // Initial adjustment B

  oled_cmd(0x16); // DIMMERCTRL - Dimmer Control
  oled_cmd(0x1F); // 32/16 for dimmer
  
  oled_cmd(0x1A); // COLCTRL - Column Output Control
  oled_cmd(0x00); // GND, GND, all off, all off (See page 72 of controller datasheet)

  oled_cmd(0x1B); // OELPERIOD1 - Setup Period 1 command
  oled_cmd(0x0F); // 4 clocks in setup period, column outputs source constant current based on dimmer and brightness

  oled_cmd(0x1C); // OELPERIOD2 - Setup Period 1 command
  oled_cmd(0x0F); // 4 clocks in setup period, column outputs source constant current based on dimmer and brightness

  oled_cmd(0x1D); // OELPERIOD3 - Setup Period 1 command
  oled_cmd(0x0F); // 4 clocks in setup period, column outputs source constant current based on dimmer and brightness

  oled_cmd(0x1E); // OELPERIOD4 - Setup Period 1 command
  oled_cmd(0xB0); // 44 clocks in setup period, column outputs at GNDL

  oled_cmd(0x11); // Dot matrix display direction
  oled_cmd(0x00); // Horizontal/Vertical inversion off, two dummy lines to precede scan line

  oled_cmd(0x12); // Dot matrix scan line select
  oled_cmd(0x3F); // 64th line

  oled_cmd(0x1F); // Odd 15th level for Greyscale lookup table
  oled_cmd(0xFF); // 255
  oled_cmd(0x20); // Odd 14th level for Greyscale lookup table
  oled_cmd(0xF2); // 242
  oled_cmd(0x21); // Odd 13th level for Greyscale lookup table
  oled_cmd(0xE3); // 227
  oled_cmd(0x22); // Odd 12th level for Greyscale lookup table
  oled_cmd(0xD7); // 215
  oled_cmd(0x23); // Odd 11th level for Greyscale lookup table
  oled_cmd(0xCA); // 202
  oled_cmd(0x24); // Odd 10th level for Greyscale lookup table
  oled_cmd(0xBE); // 190
  oled_cmd(0x25); // Odd 9th level for Greyscale lookup table
  oled_cmd(0xAA); // 170
  oled_cmd(0x26); // Odd 8th level for Greyscale lookup table
  oled_cmd(0x96); // 150
  oled_cmd(0x27); // Odd 7th level for Greyscale lookup table
  oled_cmd(0x7D); // 125
  oled_cmd(0x28); // Odd 6th level for Greyscale lookup table
  oled_cmd(0x64); // 100
  oled_cmd(0x29); // Odd 5th level for Greyscale lookup table
  oled_cmd(0x55); // 85
  oled_cmd(0x2A); // Odd 4th level for Greyscale lookup table
  oled_cmd(0x37); // 55
  oled_cmd(0x2B); // Odd 3rd level for Greyscale lookup table
  oled_cmd(0x1E); // 30
  oled_cmd(0x2C); // Odd 2nd level for Greyscale lookup table
  oled_cmd(0x0F); // 15
  oled_cmd(0x2D); // Odd 1st level for Greyscale lookup table
  oled_cmd(0x01); // 1

  oled_cmd(0x2E); // Even 15th level for Greyscale lookup table
  oled_cmd(0xFF); // 255
  oled_cmd(0x2F); // Even 14th level for Greyscale lookup table
  oled_cmd(0xF2); // 242
  oled_cmd(0x30); // Even 13th level for Greyscale lookup table
  oled_cmd(0xE3); // 227
  oled_cmd(0x31); // Even 12th level for Greyscale lookup table
  oled_cmd(0xD7); // 215
  oled_cmd(0x32); // Even 11th level for Greyscale lookup table
  oled_cmd(0xCA); // 202
  oled_cmd(0x33); // Even 10th level for Greyscale lookup table
  oled_cmd(0xBE); // 190
  oled_cmd(0x34); // Even 9th level for Greyscale lookup table
  oled_cmd(0xAA); // 170
  oled_cmd(0x35); // Even 8th level for Greyscale lookup table
  oled_cmd(0x96); // 150
  oled_cmd(0x36); // Even 7th level for Greyscale lookup table
  oled_cmd(0x7D); // 125
  oled_cmd(0x37); // Even 6th level for Greyscale lookup table
  oled_cmd(0x64); // 100
  oled_cmd(0x38); // Even 5th level for Greyscale lookup table
  oled_cmd(0x55); // 85
  oled_cmd(0x39); // Even 4th level for Greyscale lookup table
  oled_cmd(0x37); // 55
  oled_cmd(0x3A); // Even 3rd level for Greyscale lookup table
  oled_cmd(0x1E); // 30
  oled_cmd(0x3B); // Even 2nd level for Greyscale lookup table
  oled_cmd(0x0F); // 15
  oled_cmd(0x3C); // Even 1st level for Greyscale lookup table
  oled_cmd(0x01); // 1

  oled_cmd(0x17); // ROWDRVSEL - Row Driver Mode Selection
  oled_cmd(0x12); // RDIR = 64 lines to R1, Internal row driver, single scanning 72 line mode (default)

  oled_cmd(0x10); // DCTRL - Dot-Matrix Display Control
  oled_cmd(0x01); // DINV = not reversed, DALI - all off, DISPON - display on


	delay(5000L);

	oled_clr(0);    // clear the display
}

int main()
{
	delay(20000L);

	PORTD = 0xF8;	// pull-up on switches, all others off
	DDRD  = 0x0E;	// VPP and led pins output

	DDRA = 0xFF;	// data port, all outputs
	
	PORTC = 0xFE;	// all pins high
	DDRC  = 0xFE;	// all outputs

  init_OLED();    // initialize OLED module


  oled_cmd(0x16);  
  oled_cmd(0x01);

  while(1)
  {
		PORTD |= (1<<1);
		delay(50000L);
		PORTD &= ~(1<<1);
		delay(50000L);
	  bmp16((uint8_t*) logo_screen_16_level);
	  delayms(5000);

		PORTD |= (1<<1);
		delay(50000L);
		PORTD &= ~(1<<1);
		delay(50000L);
	  bmp16((uint8_t*) text_description);
	  delayms(5000);
  }

  return 0;
}


